/***************************************************************************
 *                                                                         *
 *  Copyright 2010-2011 LSI Corporation.  All rights reserved.             *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  ddriDefs.h
 *          Title:  DDR Layer Interface (DDRI) Definitions
 *     Programmer:  craigc
 *  Creation Date:  17sept09
 *
 *
 * Description
 * ------------
 *  This include file contains firmware DDR exports used
 *  by other FW modules to call DDR functions.
 *
 */

#ifndef DDRIDEFS_H_INCLUDED
#define DDRIDEFS_H_INCLUDED


/*
 **********************
 * Defines
 **********************
 */

/*
 * For Gen3 DDR3 libraries, the versioning scheme used is
 * 0xAABBCCDD, where
 * AA = Phase
 * BB = Customer / branch (250 for pre GCA)
 * CC = point release #
 * DD = developer build field (needs to be left as 0 in check-in/release code)
 *
 * Each DDR3 project (INVADER / MASTER / COBRA) has their own independent version #, based on this scheme
 *      and is also independent of SAS3FW, Cobra FW, MegaRaid, etc.
 *
 * Branches can be created as needed, likely only in the Invader project.
 *
 * For Invader, when a SAS3FW phase GCA's, the version will need to be updated (from 1.250.3.0 to 2.0.0.0, for example)
 *      and a new Invader DDR3lib release done.
 *
 * Note: the library name itself will contain the project (Cobra,Invader,Master).
 *
 */
#ifndef DDR_LIBRARY_VERSION
#define DDR_LIBRARY_VERSION   (0x01FA0300) /* This is the value to update */
#endif

/* Make sure this correctly reflects the Project (INVADER, COBRA, MASTER).  Should not ever change in a given stream*/
#define DDR_LIBRARY_PROJECT "INVADER"

/* BIST */
#define DDR_BIST_STATUS_SUCCESS 0
#define DDR_BIST_STATUS_ADDRCHECK_FAILURE 1
#define DDR_BIST_STATUS_DATACHECK_FAILURE 2
#define DDR_BIST_STATUS_INVALID_ADDRESS_ERROR 3
#define DDR_BIST_STATUS_INVALID_MEMSIZE_ERROR 4

#define DDRINIT_STATUS_OK                       (0)     /*return value of success from DDR initialization */
#define DDRINIT_STATUS_UNSUPPORTED_MEM          (3)
#define DDRINIT_BAD_MEMORY                      (4)
#define DDRINIT_STATUS_FAILED                   (0x10000000) /*Initialization failed, extended failure info is valid. */
#define DDRINIT_STATUS_FAILED_RETRY             (0x20000000) /* Initialization failed, a retry is suggested. */
/* The following items are the extended failure info. */
#define DDRINIT_STATUS_FAILED_STAGE_MASK        (0x000F0000) /* This is the step at which the failure occured. In hex. */
#define DDRINIT_STATUS_FAILED_STAGE_SHIFT       (16)         /* Use the mask then shift this many bits to find the failing step */
#define DDRINIT_STATUS_FAILED_FAULTCODE_MASK    (0x0000FFFF) /* If the extended info is not valid then the lower bytes are the faultcode. */


#define DDRI_SPD_MAX_STRING_CHARS           (24)  /* "24" must match the PERL script used to generate the spd-company-name string in the cnst struct */
#define DDRI_SPD_MAX_SERIAL_NUMBER_CHARS    (9)  /* bytes 117..125 will result in a nine-byte unique module identifier */

#define DDRI_DDR3_MIN_CAS_LATENCY           (4)   /* Min CAS latency for DDR3 per JEDEC */
#define DDRI_DDR3_MAX_CAS_LATENCY           (13)  /* Max CAS latency for DDR3 per JEDEC is 18, our HW supports 13*/

#define DDRI_DDR3_MAX_FREQUENCY_IN_MHZ      (933)  /* Max supported clock speed - DDR3-1866 - used in conversion from time(ns) to number-of-clocks */

#define DDRINIT_CALIBRATION_MEMORY_SIZE     (512)

#define DDRI_SIZE_OF_TRAINING_VALUES        (0x22300)  /*external define for IOP to know the size of the global training structure to allocate
                                                         Rounded up to nearest 0x100. */

/*defines for values passed in flags field of initDDRController */
#define DDR_INIT_SET_HALF_MODE              (0x4) /* allow the IOP to force half mode data-path if desired.
                                                        Can't be detected in SPD data */
#define DDR_INIT_USE_HIGH_TEMP_REF          (0x8) /* for high temperature environments, increase the rate of automatic refreshes */
#define DDR_INIT_FORCE_ECC_DISABLE          (0x10)/* the IOP can force the ECC byte lane disabled -
                                                       override the SPD setting - used to facilitate data corruption debug */
#define DDR_INIT_JAM_WRITE_LEVELING_VALUES  (0x20)/* skip WL algorithm, and just jam in hard coded values from some other working DIMM */
#define DDR_INIT_IGNORE_THERMAL_IRQ         (0x40)   /* for boards missing the pull-up resistor on M_EVENT_N input */
#define DDR_INIT_IGNORE_COMMAND_PARITY_IRQ  (0x80)   /* for boards missing the pull-up resistor on M_PAR input */
#define DDR_INIT_SKIP_JEDEC_SPD_REV_CHECK   (0x100)  /* allow for custom, non-JEDEC compliant DIMMs */
#define DDR_INIT_FORCE_SINGLE_RANK          (0x200)  /* allow for custom, non-JEDEC compliant DIMMs */
#define DDR_INIT_EXIT_SELF_REFRESH          (0x400)  /* to indicate that the memory is in self-refresh, so exit that mode. */
#define DDR_INIT_SKIP_READ_FIFO_TRAIN       (0x800)  /* skip read fifo algorithm, and use hard-coded values.
                                                        The values may not be optimal, this is in case read fifo is causing problems*/
#define DDR_DBG_REG_DUMP_AT_FAULT           (0x1000) /* get PHY and MC register dump during the fault */
#define DDR_INIT_ENABLE_RETRIES             (0x2000)  /* Enable return on failures to allow IOP to retry training. */
#define DDR_INIT_SIMULATE_ERROR             (0x4000)  /* Simulate training error to test error recovery at IOP level. */
#define DDR_INIT_DECREASE_RL                (0x8000)  /* Shift gate training starting sample point 1 clock earler.
                                                         Needed by certain board/memory combinations */

/*#defines for the low power mode API*/
#define  DDR_INIT_ENTRY_LP_MODE_1      (0x1) /* setup MC for low power mode 1 */
#define  DDR_INIT_ENTRY_LP_MODE_2      (0x2) /* setup MC for low power mode 2 */
#define  DDR_INIT_ENTRY_LP_MODE_3      (0x3) /* setup MC for low power mode 3 */
#define  DDR_INIT_ENTRY_LP_MODE_4      (0x4) /* setup MC for low power mode 4 */
#define  DDR_INIT_ENTRY_LP_MODE_5      (0x5) /* setup MC for low power mode 5 */

/*
 ***************************
 * Function return statuses
 ***************************
 */

/* ECC Scrub status */
#define DDR_INIT_ECC_SCRUB_STATUS_FAILURE  (1)
#define DDR_INIT_ECC_SCRUB_STATUS_SUCCESS  (0)

#define DDR_CORRECTABLE_ECC_ERROR           (0x1)    /* Indicates the correctable (Single bit) ECC errors */
#define DDR_UNCORRECTABLE_ECC_ERROR         (0x2)    /* Indicates uncorrectable (Multi bit ) ECC errors */
#define DDR_MULTIPLE_ECC_ERRORS             (0x4)    /* Indicates multiple ECC errors */
#define DDR_OUT_OF_ADDRESS_ERRORS           (0x8)    /* Indicates Out of address range  errors */
#define DDR_WRITE_DATA_PARITY_ERROR         (0x10)   /* Indicates Write data parity errors */

/*
 **********************
 * Typedefs
 **********************
 */
typedef struct _DDRI_DDR3_SPD_DATA
{
    U8 SpdDeviceSize;                            /* 0        0x00 Number of Serial PD Bytes Written / SPD Device Size / CRC Coverage */
    U8 SpdRevision;                              /* 1        0x01 SPD Revision */
    U8 DramDeviceType;                           /* 2        0x02 Key Byte / DRAM Device Type */
    U8 ModuleType;                               /* 3        0x03 Key Byte / Module Type */
    U8 SdramDensity_Banks;                       /* 4        0x04 SDRAM Density and Banks */
    U8 SdramAddressing;                          /* 5        0x05 SDRAM Addressing */
    U8 ModuleVoltage;                            /* 6        0x06 Module Nominal Voltage, VDD */
    U8 ModuleOrganization;                       /* 7        0x07 Module Organization */
    U8 ModuleMemoryBusWidth;                     /* 8        0x08 Module Memory Bus Width */
    U8 Ftb_Dividend_Divisor;                     /* 9        0x09 Fine Timebase Dividend / Divisor */
    U8 MtbDividend;                              /* 10       0x0A Medium Timebase Dividend */
    U8 MtbDivisor;                               /* 11       0x0B Medium Timebase Divisor */
    U8 tCKmin;                                   /* 12       0x0C SDRAM Minimum Cycle Time */
    U8 Reserved1;                                /* 13       0x0D Reserved */
    U8 CasLatSupported_Lsb;                      /* 14       0x0E CAS Latencies Supported, Least Significant Byte */
    U8 CasLatSupported_Msb;                      /* 15       0x0F CAS Latencies Supported, Most Significant Byte */
    U8 tAAmin;                                   /* 16       0x10 Minimum CAS Latency Time */
    U8 tWRmin;                                   /* 17       0x11 Minimum Write Recovery Time */
    U8 tRCDmin;                                  /* 18       0x12 Minimum RAS# to CAS# Delay Time */
    U8 tRRDmin;                                  /* 19       0x13 Minimum Row Active to Row Active Delay Time */
    U8 tRPmin;                                   /* 20       0x14 Minimum Row Precharge Delay Time */
    U8 tRAS_tRC3_UppNib;                         /* 21       0x15 Upper Nibbles for tRAS and tRC */
    U8 tRASmin_Lsb;                              /* 22       0x16 Minimum Active to Precharge Delay Time, Least Significant Byte */
    U8 tRCmin_Lsb;                               /* 23       0x17 Minimum Active to Active/Refresh Delay Time, Least Signif Byte */
    U8 tRFCmin_Lsb;                              /* 24       0x18 Minimum Refresh Recovery Delay Time, Least Significant Byte */
    U8 tRFCmin_Msb;                              /* 25       0x19 Minimum Refresh Recovery Delay Time, Most Significant Byte */
    U8 tWTRmin;                                  /* 26       0x1A Minimum Internal Write to Read Command Delay Time */
    U8 tRTPmin;                                  /* 27       0x1B Minimum Internal Read to Precharge Command Delay Time */
    U8 tFAW_UppNib;                              /* 28       0x1C Upper Nibble for tFAW */
    U8 tFAWmin;                                  /* 29       0x1D Minimum Four Activate Window Delay Time */
    U8 SdramOptionalFeatures;                    /* 30       0x1E SDRAM Optional Features */
    U8 SdramThermal_Refresh_Options;             /* 31       0x1F SDRAM Thermal and Refresh Options */
    U8 ModuleThermalSensor;                      /* 32       0x20 Module Thermal Sensor */
    U8 SdramDeviceType;                          /* 33       0x21 SDRAM Device Type */
    U8 tCKmin_ftb;                               /* 34       0x22 Fine Offset for tCKmin */
    U8 tAAmin_ftb;                               /* 35       0x23 Fine Offset for tAAmin */
    U8 tRCDmin_ftb;                              /* 36       0x24 Fine Offset for tRCDmin */
    U8 tRPmin_ftb;                               /* 37       0x25 Fine Offset for tRPmin */
    U8 tRCmin_ftb;                               /* 38       0x26 Fine Offset for tRCmin */
    U8 Reserved2[21];                            /* 39-59    0x27-0x3B Reserved, General Section */
    U8 ModuleSpecificSection[57];                /* 60-116   0x3C-0x74 Module Type Specific Section, Indexed by Key Byte 3 */
    U8 ModuleId_ManufacturersJedecIdCode[2];     /* 117-118  0x75-0x76 Module ID: Module Manufacturers JEDEC ID Code */
    U8 ModuleId_ManufacturingLocation;           /* 119      0x77 Module ID: Module Manufacturing Location */
    U8 ModuleId_ManufacturingDate[2];            /* 120-121  0x78-0x79 Module ID: Module Manufacturing Date */
    U8 ModuleId_SerialNumber[4];                 /* 122-125  0x7A-0x7D Module ID: Module Serial Number */
    U8 CrcCode[2];                               /* 126-127  0x7E-0x7F Cyclical Redundancy Code */
    U8 ModulePartNumber[18];                     /* 128-145  0x80-0x91 Module Part Number */
    U8 ModuleRevisionCode[2];                    /* 146-147  0x92-0x93 Module Revision Code */
    U8 ManufacturesJedecIdCode[2];               /* 148-149  0x94-0x95 DRAM Manufacturers JEDEC ID Code */
    U8 ManufacturesSpecificData[26];             /* 150-175  0x96-0xAF Manufacturers Specific Data */
    U8 Reserved3[80];                            /* 176-255  0xB0-0xFF Open for customer use */
} DDRI_DDR3_SPD_DATA, * PTR_DDRI_DDR3_SPD_DATA;


#define DDRI_SPD_VOLTAGE_1P5V           (1)  /* 1.5v  */
#define DDRI_SPD_VOLTAGE_1P35V          (2)  /* 1.35v */
#define DDRI_SPD_VOLTAGE_1P2V           (3)  /* 1.2v  */

#define DDRI_PSM_PB_NO_PREFETCH         (1)
#define DDRI_PSM_PB_BASE_PREFETCH       (2)
#define DDRI_PSM_PB_64B_PREFETCH        (3)
#define DDRI_PSM_PB_128B_PREFETCH       (4)

#define DDRI_RTTNOM_DISABLED            (1)
#define DDRI_RTTNOM_60_OHM              (2)
#define DDRI_RTTNOM_120_OHM             (3)
#define DDRI_RTTNOM_40_OHM              (4)
#define DDRI_RTTNOM_20_OHM              (5)
#define DDRI_RTTNOM_30_OHM              (6)

#define DDRI_ODS_40_OHM                 (1)
#define DDRI_ODS_34_OHM                 (2)

#define DDRI_RTTWR_DISABLED             (1)
#define DDRI_RTTWR_RQZ_DIV_4            (2)
#define DDRI_RTTWR_RQZ_DIV_2            (3)

#define DDRI_SILICON_NO_REV_SPECIFIED   (0)  /* let it auto detect     */
#define DDRI_SILICON_REV_TBOLT_A0       (1)  /* Tbolt A0       */
#define DDRI_SILICON_REV_TBOLT_A1       (2)  /* Tbolt A1 - via8 only metal spin to fix jitter - can't auto detect because HW design didn't change the registers */
#define DDRI_SILICON_REV_TBOLT_B0       (3)  /* Tbolt B0 (and later)    */
#define DDRI_SILICON_REV_INVADER_A0     (5)  /* Invader A0     */
#define DDRI_SILICON_REV_INVADER_B0     (6)  /* Invader B0     */
    /*                               7?     Invader C0 ??? */
#define DDRI_SILICON_REV_COBRA_A0       (8)  /* Cobra A0       */
    /*                               9?     Cobra B0 ???   */


typedef struct _DDRINIT_MEM_INFO
{
    /*
     * This structure must be zero'ed and initialized before calling init-ddr-controller
     */
    U8    *ptrCalibMem;             /* Base of memory used for training, up to size DDRINIT_CALIBRATION_MEMORY_SIZE -
                                       contents will be destroyed during training process */
    U8    *ptrCalibMemSecondRank;   /* same as above, but for second rank (when there are two ranks in the DIMM) */
    U8    *ptrPatBuf;               /* Place to DMA result from  local-mem --> DDR       */
    U8    *ptrCmpBuf;               /* Place to DMA result from        DDR --> local-mem */
    U32   MemCtlRegs;               /* Base of memory controller registers */
    U32   MemDcrRegs;               /* Base DCR address of memory controller regs */
    U8    RttNom;                   /* Nominal termination */
    U8    RttWr;                    /* dynamic ODT on writes */
    U8    DimmOutputDriveStrength;  /* DIMM output drive strength */
    U8    DebugForceSiliconRev;     /* force a specific silicon revision - override the auto detect logic */
    U32   *ptrTrainingData;         /* location for the library to put global training data*/
    U8    *ptrCalibMemNoCache;      /* Same as ptrCalibMem, but via non-cached access, for read fifo training */
    U8    *ptrCalibMemSecondRankNoCache;    /* same as above, but for second rank (when there are two ranks in the DIMM) */

} DDRINIT_MEM_INFO;


#define ECC_SYNDROME_NO_ERROR       (1)
#define ECC_SYNDROME_BAD_DATABIT    (2)
#define ECC_SYNDROME_BAD_CHECKBIT   (3)
#define ECC_SYNDROME_MULTIBIT_ERROR (4)

typedef struct _DDRI_DDR3_ECC_ERROR_INFO
{
    U32 dataWordHigh;
    U32 dataWordLow;
    U32 addrHigh;
    U32 addrLow;
    U32 sourceId;
    U32 syndrome; /* indicates ECC_SYNDROME_NO_ERROR ECC_SYNDROME_BAD_DATABIT etc */
    U32 badBit;   /* only valid when syndrome is bad databit/checkbit */
} DDRI_DDR3_ECC_ERROR_INFO, * PTR_DDRI_DDR3_ECC_ERROR_INFO;

typedef struct _DDRI_DDR3_ECC_INFO
{
    DDRI_DDR3_ECC_ERROR_INFO correctableInfo;
    DDRI_DDR3_ECC_ERROR_INFO uncorrectableInfo;
    U32 intStatus; /* for LSI reference only*/
    U32 eccStatus; /* shows DDR_CORRECTABLE_ECC_ERROR,  DDR_UNCORRECTABLE_ECC_ERROR,
                    * DDR_MULTIPLE_ECC_ERRORS (any combination of the three) */
} DDRI_DDR3_ECC_INFO, * PTR_DDRI_DDR3_ECC_INFO;

#endif  /* if not defined ddridefs-h-included */

